/*
 * Cell.C
 *
 * Copyright (C) 1999, Thierry Matthey <matthey@ii.uib.no>
 *
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 */

#include "Cell.h"

//_________________________________________________________________ Cell

Cell::Cell()  
:myIndex(-1,-1)  
{
  myPath = NULL;
  myParticles = new FastList<Particle>(0);
  myWeight = 1;
  usedByReset();
#ifdef HAVE_MPI
#ifdef HAVE_ONE 
  myShmPtr = NULL;
#endif
#endif
}

Cell::Cell(CellIndex index,List<CellIndex>* path)  
:myIndex(index)  
{
  myPath = path;
  myParticles = new FastList<Particle>(0);
  myWeight = 1;
  usedByReset();
#ifdef HAVE_MPI
#ifdef HAVE_ONE 
  myShmPtr = NULL;
#endif
#endif
}

Cell::~Cell(){
  delete myParticles;
  delete myPath;
}

void Cell::setPath(List<CellIndex>* path){
  if(myPath != NULL && path != myPath)
    delete path;
  myPath = path;
}

void Cell::usedByReset(){
  for(long k=0;k<max_neighbours;k++)
     myUsedBy[k] = -1;
}

void Cell::usedByAdd(int id){

  if(id == myOwnerId)
    return;

  for(long k=0;k<max_neighbours;k++){
    if (myUsedBy[k] == id)
      return;
    else if (myUsedBy[k] == -1){
      myUsedBy[k] = id;
      return;
    }
  }
}

bool Cell::usedById(int id) const{
  for(long k=0;k<max_neighbours;k++){
    if (myUsedBy[k] == -1)
      return false;
    if(myUsedBy[k] == id)
      return true;
  }
  return false;
}
