/*
 * CellIndex.C
 *
 * Copyright (C) 1999, Thierry Matthey <matthey@ii.uib.no>
 *
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 */

#include "CellIndex.h"

//_________________________________________________________________ CellIndex

CellIndex::CellIndex()
{ 
  myVec[0] = myVec[1] /*= myVec[2]*/ = 0; 
}

CellIndex::CellIndex(long i, long j /*, long k*/) 
{ 
  myVec[0] = i; myVec[1] = j; //myVec[2] = k; 
}

CellIndex::CellIndex(const CellIndex& vec)
{ 
  myVec[0] = vec.myVec[0]; 
  myVec[1] = vec.myVec[1]; 
  //myVec[2] = vec.myVec[2]; 
}

CellIndex& CellIndex::operator=(const CellIndex& vec)
{
  myVec[0] = vec.myVec[0]; 
  myVec[1] = vec.myVec[1]; 
  //myVec[2] = vec.myVec[2];

  return *this;
}

CellIndex& CellIndex::operator+=(const CellIndex& vec)
{
  myVec[0] += vec.myVec[0];  
  myVec[1] += vec.myVec[1];  
  //myVec[2] += vec.myVec[2];

  return *this;
}

CellIndex& CellIndex::operator-=(const CellIndex& vec)
{
  myVec[0] -= vec.myVec[0];  
  myVec[1] -= vec.myVec[1];  
  //myVec[2] -= vec.myVec[2];

  return *this;
}

CellIndex CellIndex::operator-(const CellIndex& vec) const
{
  return CellIndex(myVec[0] - vec.myVec[0], 
		   myVec[1] - vec.myVec[1] //, 
		   //myVec[2] - vec.myVec[2]
		   );
}

int CellIndex::operator==(const CellIndex& vec) const
{
  return (myVec[0] == vec.myVec[0] && 
	  myVec[1] == vec.myVec[1] /*&& 
					    myVec[2] == vec.myVec[2]*/);
}

int CellIndex::operator!=(const CellIndex& vec) const
{
  return (myVec[0] != vec.myVec[0] || 
	  myVec[1] != vec.myVec[1] /*|| 
					     myVec[2] != vec.myVec[2]*/);
}

#ifdef HAVE_MPI
void  CellIndex::build_derived_type(MPI_Datatype *handle){

  int block_lengths[] = { 2, 1};
  MPI_Datatype types[] = { MPI_LONG, MPI_UB }; 
  MPI_Aint displacements[2];
  
  displacements[0] = 0;
  displacements[1] = sizeof(CellIndex);
  
  MPI_Type_struct(2, block_lengths, displacements, types, handle);
  MPI_Type_commit(handle);  

}
#endif

ostream& operator<<(ostream& OS, CellIndex & index){
  OS << "["<<index.myVec[0]<<","<<index.myVec[1]<<"]";
  return OS;
}
