/*
 * CellIndex.C
 *
 * Copyright (C) 1999, Thierry Matthey <matthey@ii.uib.no>
 *
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 */

#ifndef _CELLINDEX_H
#define _CELLINDEX_H

#ifdef NO_ANSI_CPP
#include <iostream.h>
#else
#include <iostream>
using std::cerr;
using std::ostream;
#endif

#ifdef HAVE_MPI
#include <mpi.h>
#endif

//_________________________________________________________________ CellIndex

class CellIndex {

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Constructors, destructors, assignment
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 public:
  CellIndex();
  CellIndex(long i, long j /*,long k*/); 
  CellIndex(const CellIndex& vec);
  CellIndex& operator=(const CellIndex& vec);
  long& operator[](int index);
  long  operator[](int index) const;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// New methods of class CellIndex
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 public:
  long& i();
  long  i() const;
  long& j();
  long  j() const;
  //long& k();
  //long  k() const;

  CellIndex& operator+=(const CellIndex& vec);
  CellIndex& operator-=(const CellIndex& vec); 
  CellIndex operator-(const CellIndex& vec) const;  // CellIndex vector subtraction
  int operator==(const CellIndex& vec) const;
  int operator!=(const CellIndex& vec) const;
  CellIndex operator-() const;
  friend ostream& operator<<(ostream& OS, CellIndex & index);
#ifdef HAVE_MPI
  static void build_derived_type(MPI_Datatype *handle);
#endif

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// My data members
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 private:
  long myVec[2];
};

//______________________________________________________________________ INLINES

inline long& CellIndex::operator[](int index)
{ 
  if (index<0 || index>1){
    cerr << "[CellIndex::operator[] ] index out of range\n";
    return myVec[0];
  }

  return myVec[index]; 
}

inline long CellIndex::operator[](int index) const 
{ 
  if (index<0 || index>1){
    cerr << "[CellIndex::operator[] const] index out of range\n";
    return myVec[0];
  }
  return myVec[index]; 
}

inline long& CellIndex::i()
{
  return myVec[0];
}

inline long CellIndex::i() const
{
  return myVec[0];
}

inline long& CellIndex::j()
{
  return myVec[1];
}

inline long CellIndex::j() const
{
  return myVec[1];
}
/*
inline long& CellIndex::k()
{
  return myVec[2];
}

inline long CellIndex::k() const
{
  return myVec[2];
}
*/
#endif
















































