/*
 * Copyright (c) 1991 Stanford University
 * Copyright (c) 1991 Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * Stanford and Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Stanford and Silicon Graphics.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 *
 * IN NO EVENT SHALL STANFORD OR SILICON GRAPHICS BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 *
 * ---------------------------------------------------------------------
 *  $Id: FastList.C,v 1.8 1996/02/16 12:52:42 streit Exp $
 * ---------------------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
//_______________________________________________ Support routines for lists

#ifndef TEMPLATE_IS_INCLUDE_FROM_H
static long FastListImpl_best_new_sizes[] = {
    48, 112, 240, 496, 1008, 2032, 4080, 8176,
    16368, 32752, 65520, 131056, 262128, 524272, 1048560,
    2097136, 4194288, 8388592, 16777200, 33554416, 67108848
};

long FastListImpl_best_new_count(long count, long size) 
{
  for (int i = 0; i < int(sizeof(FastListImpl_best_new_sizes)/sizeof(long)); i++) {
    if (count * size < FastListImpl_best_new_sizes[i]) {
      return FastListImpl_best_new_sizes[i] / size;
    }
  }
  return count;
}
#endif // TEMPLATE_IS_INCLUDE_FROM_H

/*
 * GNU compiler has a pretty bad handling of templates...
 */
#if defined( TEMPLATE_IS_INCLUDE_FROM_H) || !defined(TEMPLATE_IN_HEADER)
// Only compile this file if either not GNU compiler running,
// or, in case of GNU compiler, files is included from header.

#ifdef TEMPLATE_IN_HEADER
# define INLINE inline
#else
# include "FastList.h"
# define INLINE
#endif  // TEMPLATE_IN_HEADER

//___________________________________________________________________ FastList<T>

template <class T>
INLINE FastList<T>::FastList()
  : myItems(NULL),mySize(NULL), myCount(NULL),myPtr(NULL) 
{}

template <class T>
INLINE FastList<T>::FastList(long size)
{ 
  if (size > 0) {
    long n = FastListImpl_best_new_count(size, sizeof(T)); 
    myPtr   = mymalloc(T,n);
    if(myPtr == NULL)
      cerr <<"Out of Memory!\n";
    mySize  = (long*)myPtr;
    myCount = (long*)myPtr+1;
    myItems = (T*)((long*)myPtr+2);
    (*myCount) = 0;
    (*mySize)  = n;
  } 
  else { 
    myPtr   = NULL;
    mySize  = NULL; 
    myCount = NULL; 
    myItems = NULL; 
  } 
}

template <class T>
INLINE FastList<T>::FastList(const FastList<T>& list)
: myItems(NULL),mySize(NULL), myCount(NULL),myPtr(NULL) 
{
  long count = list.count();
  for (long i=0; i<count; i++)
    add(list.item(i));
}

template <class T>
INLINE FastList<T>::FastList(long size, const T* items)
{
  if (size > 0) {
    long n = FastListImpl_best_new_count(size, sizeof(T)); 
    myPtr   = mymalloc(T,n);
    mySize  = (long*)myPtr;
    myCount = (long*)myPtr+1;
    myItems    = (T*)((long*)myPtr+2);
    (*myCount) = size;
    (*mySize)  = n;
    for (long i=0; i<size; i++)
      myItems[i] = items[i];
  } 
  else { 
    myPtr   = NULL;
    mySize  = NULL; 
    myCount = NULL; 
    myItems = NULL; 
  } 
}

template <class T>
INLINE FastList<T>::~FastList()
{
  myfree(myPtr);
}

template <class T>
INLINE FastList<T>& FastList<T>::operator=(const FastList<T>& list)
{
  if (this == &list)
    return *this;
  
  removeAll();
  for (register long i=0; i<list.count(); i++)
    add(list.item(i));

  return *this;
}

template <class T>
INLINE T& FastList<T>::item(long index) const
{
  //
  // index out of range?
  //
  if (index < 0 || index >= count()) {
    cerr << "[FastList<T>::item] index " << index << " out of range" << "\n";
    return myItems[0];
  }
     
  return myItems[index]; 
}

template <class T>
INLINE void FastList<T>::remove(long index) 
{ 
  if (index >= 0 && index < count()) { 
    if (index <(*myCount)-1) { 
      myItems[index] = myItems[(*myCount)-1];
    }
    (*myCount)--; 
  } 
}
 
template <class T>
INLINE long FastList<T>::removeItem(const T& item) 
{ 
  long index;
  for(index=0;index<count();index++)
    if ( memcmp(&(myItems[index]), &item, sizeof(T)) == 0 )
       break;

  if (index >= 0 && index < count()) { 
    if (index <(*myCount)-1) { 
      myItems[index] = myItems[(*myCount)-1];
    }
    (*myCount)--; 
  }
  else
    index = -1;
  return index;
} 

template <class T>
INLINE void FastList<T>::add(const T& item) 
{ 
  atLeastSize(count() +1);
  myItems[(*myCount)] = item; 
  (*myCount)++; 

} 

template <class T>
INLINE bool FastList<T>::addUnique(const T& item) 
{ 
  long i;
  for(i=0;i<count();i++)
    if ( memcmp(&(myItems[i]), &item, sizeof(T)) == 0 )
      return false;

  add(item);
  return true;
} 

template <class T>
INLINE void FastList<T>::add(long size, const T* items) 
{ 

  if(size == 0)
    return;

  atLeastSize(size+count());

  long i;
  for(i=0;i<size;i++)
    myItems[i+(*myCount)] = items[i];
  (*myCount) += size;

}
 
template <class T>
INLINE void FastList<T>::addFast(long size, const T* items) 
{ 
  if(size == 0)
    return;

  atLeastSize(size+count());
  bcopy(items,&myItems[(*myCount)],size*sizeof(T));
  (*myCount) +=size;
} 


template <class T>
INLINE void FastList<T>::setItems(long size, const T* items) 
{ 
  atLeastSize(size);

  for (int i = 0; i < size; ++i) { 
    myItems[i] = items[i]; 
  }
  (*myCount) = size;
} 

template <class T>
INLINE T* FastList<T>::setItems(long size) 
{ 
  atLeastSize(size);
  if(myCount != NULL)
    (*myCount) = size;
  return myItems;
} 

template <class T>
INLINE void FastList<T>::atLeastSize(long leastsize) 
{ 
  if(leastsize > size()){
    T*  items = myItems;
    void* ptr = myPtr;
    long l = count();
    long n = FastListImpl_best_new_count(leastsize, sizeof(T)); 
    myPtr   = mymalloc(T,n);
    if(myPtr == NULL)
      cerr <<"Out of Memory!\n";
    mySize  = (long*)myPtr;
    myCount = (long*)myPtr+1;
    myItems = (T*)((long*)myPtr+2);
    *myCount = l;
    *mySize  = n;
    for (long i=0; i<l; i++)
      myItems[i] = items[i];

    myfree(ptr);
  }
} 

template <class T>
INLINE void FastList<T>::reduce() 
{ 

  if(count() == 0){
    if(myPtr != NULL)
      myfree(myPtr);
    myPtr   = NULL;
    myCount = NULL;
    mySize  = NULL;
    return;
  }

  T*  items = myItems;
  void* ptr = myPtr;
  long l = count();
  long n = FastListImpl_best_new_count(l, sizeof(T)); 

  if(n < size()){
    myPtr   = mymalloc(T,n);
    if(myPtr == NULL)
      cerr <<"Out of Memory!\n";
    mySize  = (long*)myPtr;
    myCount = (long*)myPtr+1;
    myItems = (T*)((long*)myPtr+2);
    *myCount = l;
    *mySize  = n;
    for (long i=0; i<l; i++)
      myItems[i] = items[i];

    myfree(ptr);
  }
} 

template <class T>
INLINE T* FastList<T>::getItems() const
{ 
  return myItems;
} 

template <class T>
INLINE void FastList<T>::removeAll()
{
  if(myCount != NULL)
    (*myCount) = 0;
}

template <class T>
INLINE void FastList<T>::test() const
{ 
  cerr <<"FastList<T>::test()\n"
       <<"myPtr   :"<<myPtr<<"\n"
       <<"mySize  :"<<myPtr<<", "<<*mySize<<"\n"
       <<"myCount :"<<myPtr<<", "<<*myCount<<"\n";
  for(long i=0;i<(*mySize)*sizeof(T)+2*sizeof(long);i++)
    cerr<<"myPtr["<<i<<"]:"<<(int)(((char*)myPtr)[i])<<"\n";

} 

//  template <class T>
//  ostream& operator<<(ostream& OS, FastList<T> & mylist){
//    long i;
//    for(i=0;i<mylist.myCount;i++){
//      OS << mylist.myItems[i];
//    }
//    return OS;
//  }

//________________________________________________________________ FastListItr<T>

template <class T>
INLINE void FastListItr<T>::first() 
{ 
  myCurItem = 0;
  //  myFastListCount = myList->count(); 
  myListCount = myList->count(); 
} 

template <class T>
INLINE void FastListItr<T>::next() 
{ 
  ++myCurItem;
} 

template <class T>
INLINE int FastListItr<T>::isDone() const 
{ 
  return !(myCurItem < myListCount); 
}
 
template <class T>
INLINE T& FastListItr<T>::curItem() const 
{ 
  return myList->item(myCurItem);
} 

template <class T>
INLINE FastListItr<T>::FastListItr(const FastList<T>& list) 
{ 
  myList = &list; 
  myCurItem = 0; 
  myListCount = myList->count(); 
}

#undef INLINE  
#endif // defined( TEMPLATE_IS_INCLUDE_FROM_H) || !defined(TEMPLATE_IN_HEADER)
