/*
 * Copyright (c) 1987, 1988, 1989, 1990, 1991 Stanford University
 * Copyright (c) 1991 Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * Stanford and Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Stanford and Silicon Graphics.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 *
 * IN NO EVENT SHALL STANFORD OR SILICON GRAPHICS BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 * 
 * Generic list implemented as dynamic array
 *
 * ----------------------------------------------------------------------
 *  $Id: FastList.h,v 1.18 1996/02/16 12:52:43 streit Exp $
 * ----------------------------------------------------------------------
 */

#ifndef _FastList_H
#define _FastList_H

#include "global.h"

extern long FastListImpl_best_new_count(long count, long size);
  
//___________________________________________________________________ FastList<T>

template <class T>
class FastList { 
public: 
  FastList();
  FastList(long size);
  FastList(const FastList<T>& list);
  FastList(long size, const T* items);

  ~FastList();

  FastList<T>& operator=(const FastList<T>& list);

  long count() const  { return (myCount == NULL ? 0 : *myCount); } 
  long size()  const  { return (mySize  == NULL ? 0 : *mySize); } 
  T& item(long index) const;
 
  void add(const T& item);
  bool addUnique(const T& item);
  void add(long size, const T* items);
  void addFast(long size, const T* items);
  void setItems(long size, const T* items);
  T* setItems(long size);
  T* getItems() const;
  void remove(long index);
  long removeItem(const T& item);
  void removeAll();
  void atLeastSize(long leastsize);
  void reduce();
  void* getPtr() const {return myPtr;};
  //  friend ostream& operator<<(ostream& OS, FastList<T> & mylist);

  void test() const;

private:
  void* myPtr;
  long* mySize; 
  long* myCount;
  T*    myItems;  

}; 


//________________________________________________________________ FastListItr<T>
//
// Non robust Iterator over List
//
template <class T>
class FastListItr { 
public: 
  FastListItr(const FastList<T>& list); 

  void first();
  void next(); 
  int isDone() const; 
  T& curItem() const; 
  
private: 
  const FastList<T>* myList; 
  long myCurItem; 
  long myListCount; 
};

/*
 * GNU compiler has a pretty bad handling of templates...
 */
#ifdef TEMPLATE_IN_HEADER
#define TEMPLATE_IS_INCLUDE_FROM_H
#include "FastList.C"
#endif // TEMPLATE_IN_HEADER
 
#endif // _FastList_H
