/*
 * Force.h
 *
 * Copyright (C) 1999-2000, Thierry Matthey <matthey@ii.uib.no>
 *
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 */

#ifndef _FORCE_H
#define _FORCE_H

#ifdef NO_ANSI_CPP
#include <iostream.h>
#include <fstream.h>
#else
#include <iostream>
#include <fstream>
using std::cerr;
using std::ifstream;
using std::ostream;
using std::ios;
#endif

#include "global.h"
class Particle;
//_________________________________________________________________ Force

class Force {

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Constructors, destructors, assignment
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 public:
  Force();

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// New methods of class Force
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  Force* copy() const;
  bool calc(Particle& a, Particle& b, Real& epot, Real bx=0.0, Real by=0);
  void readParameters(ifstream& parameters);

  void setRCut(Real rcut);
  Real getMin() const;

 private:
  void nextLine(ifstream& in);

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// My data members
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 public:
  Real fx;
  Real fy;

 private:

  Real myRCut;
  Real myRCutSq;
  Real myMin;

  bool used   [max_NTypes][max_NTypes];
  Real con    [max_NTypes][max_NTypes];
  Real con4   [max_NTypes][max_NTypes];
  Real alf    [max_NTypes][max_NTypes];
  Real alfp   [max_NTypes][max_NTypes];
  long nalf   [max_NTypes][max_NTypes];
  Real bet    [max_NTypes][max_NTypes];
  Real betp   [max_NTypes][max_NTypes];
  long nbet   [max_NTypes][max_NTypes];
  Real zero   [max_NTypes][max_NTypes];
  Real min    [max_NTypes][max_NTypes];
  Real epsilon[max_NTypes][max_NTypes];
 
};
//______________________________________________________________________ INLINES

inline Real Force::getMin() const{
  return myMin;
}


#endif // _FORCE_H
















































