/*
 * Particle.h
 *
 * Copyright (C) 1999, Thierry Matthey <matthey@ii.uib.no>
 *
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 */

#include "Particle.h"

#include <stdlib.h>
#include <stdio.h>

//_________________________________________________________________ Particle

Particle::Particle(){
  tag    = 0;
  x      = 0;
  y      = 0;
  vx     = 0;
  vy     = 0;
  fxa    = 0;
  fya    = 0;
  epot   = 0;
  //theta  = 0;
  //w      = 0;
  mass   = 0;
  radius = 1;
  i      = 0;
}

Particle::Particle(Real xin, Real yin){
  tag    = 0;
  x      = xin;
  y      = yin;
  vx     = 0;
  vy     = 0;
  fxa    = 0;
  fya    = 0;
  epot   = 0;
  //theta  = 0;
  //w      = 0;
  mass   = 0;
  radius = 1;
  i      = 0;
}

Particle::Particle(Real xin, Real yin, Real r, Real inmass, short intag, long ini){
  tag    = intag;
  x      = xin;
  y      = yin;
  vx     = 0;
  vy     = 0;
  fxa    = 0;
  fya    = 0;
  epot   = 0;
  //theta  = 0;
  //w      = 0;
  mass   = inmass;
  radius = r;
  i      = ini;
}

Particle::Particle(Real xin, Real yin, Real uin, Real vin, Real r, Real inmass, short intag, long ini){
  tag    = intag;
  x      = xin;
  y      = yin;
  vx     = uin;
  vy     = vin;
  fxa    = 0;
  fya    = 0;
  epot   = 0;
  //theta  = 0;
  //w      = 0;
  mass   = inmass;
  radius = r;
  i      = ini;
}

ostream& operator<<(ostream& OS, Particle & par){
  char num[buffer_len];
  sprintf(num,"%08d",(int)par.i);
  //OS.precision(16);
  OS << num <<","<< par.tag <<" : "<<"r(" << par.x << "," << par.y << "), v(" << par.vx << "," << par.vy << "), f(" << par.fxa << "," << par.fya << "), mass("<<par.mass<<")\n";

  return OS;
}
#ifdef HAVE_MPI
void  Particle::build_derived_type(MPI_Datatype *handle){

  //Not very nice !!!
  int block_lengths[] = { sizeof(Particle), 1};
  MPI_Datatype types[] = { MPI_BYTE, MPI_UB }; 
  MPI_Aint displacements[2];
  
  displacements[0] = 0;
  displacements[1] = sizeof(Particle);
  
  MPI_Type_struct(2, block_lengths, displacements, types, handle);
  MPI_Type_commit(handle);  

}
#endif
