/*
 * Particle.h
 *
 * Copyright (C) 1999, Thierry Matthey <matthey@ii.uib.no>
 *
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 */

#ifndef _PARTICLE_H
#define _PARTICLE_H

#ifdef NO_ANSI_CPP
#include <iostream.h>
#else
#include <iostream>
using std::cerr;
using std::ostream;
#endif

#ifdef HAVE_MPI
#include <mpi.h>
#endif

#include "global.h"

//_________________________________________________________________ Particle

class Particle {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Constructors, destructors, assignment
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 public:
  Particle();
  Particle(Real x, Real y);
  Particle(Real x, Real y, Real r, Real inmass, short tag, long i);
  Particle(Real x, Real y, Real u, Real v, Real r, Real inmass, short tag, long i);
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// New methods of class Particle
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#ifdef HAVE_MPI
  static void build_derived_type(MPI_Datatype *handle);
#endif

  friend ostream& operator<<(ostream&, Particle&);

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// My data members
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 public:
  short tag;    // Type of particle

  Real x;       // Position
  Real y;	
  Real vx;      // Velocity
  Real vy;
  Real fxa;     // Force
  Real fya;
  Real epot;
  //Real theta;   // Angle
  //Real w;       // Angle velocity

  Real mass;    // Mass
  Real radius;  // Radius

  long i;       // Internal number

};


#endif
















































