/*
 * StartWorld.h
 *
 * Copyright (C) 1999, Thierry Matthey <matthey@ii.uib.no>
 *
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 */

#ifndef _STARTWOLRD_H
#define _STARTWOLRD_H

#ifdef NO_ANSI_CPP
#include <iostream.h>
#include <fstream.h>
#else
#include <iostream>
#include <fstream>
using std::cerr;
using std::ifstream;
using std::ostream;
using std::ios;
#endif

#include "global.h"

class Force;
class Universe;

//_________________________________________________________________ StartWorld

class StartWorld {

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Constructors, destructors, assignment
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 public:
  StartWorld(char *filename, Force* force);
  ~StartWorld(); 
 private:
  StartWorld& operator=(const StartWorld&);
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// New methods of class StartWorld
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 public:
  Real getLeft() const;
  Real getRight() const;
  Real getTop() const;
  Real getBottom() const;

  Real getRCut() const;
  Real getTStop() const;
  Real getPullStart() const;
  Real getPullStop() const;
  Real getDt() const;
  long getNStop() const;
  Real getDj() const;
  long getNumber() const;
  Real getTemp() const;
  Real getMaxRadius() const;
  char* getOutFilename();
  char* getInFilename();


  Force* getForce() const;

  void addParticles(Universe* universe);

 private:
  void nextLine(ifstream& in);

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// My data members
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 private:
  Real myLeft;
  Real myRight;
  Real myTop;
  Real myBottom;
  Real myReadLeft;
  Real myReadRight;
  Real myReadTop;
  Real myReadBottom;
  char myOutFilename[buffer_len];
  char myInFilename[buffer_len];
  Real myTStop;
  Real myDt;
  Real myDj;
  long myNStop;
  Real myMaxRadius;

  Real myRCut;
  Real myTemp;

  long myNTypes;
  long myNumber;

  Real myProcent[max_NTypes];
  Real myMass[max_NTypes];
  Real myRadius[max_NTypes];

  Real myPull;
  Real myPullStop;
  Real myPullStart;

  Force* myForce;

};

//______________________________________________________________________ INLINES

inline char* StartWorld::getOutFilename() {
  return myOutFilename;
}
inline char* StartWorld::getInFilename() {
  return myInFilename;
}
inline Force* StartWorld::getForce() const{
  return myForce;
}
inline Real StartWorld::getRCut() const{
  return myRCut;
}
inline Real StartWorld::getTemp() const{
  return myTemp;
}
inline long StartWorld::getNStop() const{
  return myNStop;
}
inline long StartWorld::getNumber() const{
  return myNumber;
}
inline Real StartWorld::getDj() const{
  return myDj;
}
inline Real StartWorld::getDt() const{
  return myDt;
}
inline Real StartWorld::getTStop() const{
  return myTStop;
}
inline Real StartWorld::getPullStop() const{
  return myPullStop;
}
inline Real StartWorld::getPullStart() const{
  return myPullStart;
}
inline Real StartWorld::getMaxRadius() const{
  return myMaxRadius;
}

inline Real StartWorld::getLeft() const{
  return myLeft;
}
inline Real StartWorld::getRight() const{
  return myRight;
}
inline Real StartWorld::getBottom() const{
  return myBottom;
}
inline Real StartWorld::getTop() const{
  return myTop;
}



#endif // _STARTWOLRD_H
















































