/*
 * display.h
 *
 * Copyright (C) 1997-99, Thierry Matthey <matthey@ii.uib.no>
 *
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 */

#ifndef _DISPLAY_H
#define _DISPLAY_H

#include "global.h"
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>


class Universe;

//_________________________________________________________________ display

static const long SAVE_ONCE    = 1<<0;
static const long SAVE_ALLWAYS = 1<<1;
static const long SAVE_AS_RAS  = 1<<2;
static const long SAVE_AS_PBM  = 1<<3;
static const long SAVE_AS_PS   = 1<<4;
static const long SAVE_AS_PPM  = 1<<5;
static const long SAVE_AS_GIF  = 1<<6;

static const long DEFAULT_WIDTH  = 800;
static const long DEFAULT_HEIGHT = 600;
static const long MIN_WIDTH  = 200;
static const long MIN_HEIGHT = 150;
static const long MAX_XDISPLAY = 20000;
static const Real SCALE_OF_OUTPUT = 1.6;

static const long  num_cell_max = 256;

/* Structur for the datas for the output */
typedef struct {
  /* X11 */
  Display* display;
  Window window;
  Pixmap pixmap;  /* The drawbuffer */
  XSizeHints size_hints;
  GC gc;
  XFontStruct *font_info;
  int screen;

  /* Window size */
  Real left,right;   /* Size of the input */
  Real bottom,top;
  int width,height;  /* Size of the outputwindow */
  Real a,bx,by;      /* Scalefactors */

  /* Internal data */
  bool flush;        /* If FALSE then dont flush after writeDisplay(); */
  bool update;    
  bool init;
  int  savepic;
  unsigned long background; /* Color of the background */
  unsigned long foreground; /* Color of the foreground */

  /* X11 Structs */
  XArc *arcs;
  XSegment *segments;

  char* out;
  char* in;
  unsigned long user; /* The actual step */
} OutputWindow; 

/* Opening a X11 window and creating an OutputWindow structur to save datas for output */
OutputWindow* openDisplay(Universe* universe, int width, int height, bool flush, int savepic, int argc, char* argv[],char* in, char* out);

/* Closing the outputwindow and freeing the allocated memeory */
void closeDisplay(OutputWindow* ow);

/* Flushing the drawing from the backstorebuffer onto the visible window */
void flushDisplay(OutputWindow* ow);

/* Drawing the datas into the backstorebuffer */
void writeDisplay(OutputWindow* ow, Universe* universe);

/* Refreshing the drawing */
void refreshDisplay(OutputWindow* ow);

bool resizeDisplay(OutputWindow* ow, int width, int height);
#endif




















