/*
 * global.C
 *
 * Copyright (C) 1999-2000, Thierry Matthey <matthey@ii.uib.no>
 *
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 */


#include "global.h"
#include <sys/timeb.h>
#include <time.h>

//_________________________________________________________________ global

#ifdef HAVE_MPI
extern double mytime(){
  timerCounter++;
  return MPI_Wtime();
}
#else
#ifdef __sgi
struct timespec tp;
double mytime(){
  timerCounter++;
  clock_gettime(CLOCK_SGI_CYCLE, &tp);
  return tp.tv_sec+((double)tp.tv_nsec)*1.e-9;
}
#else
struct timeb tp;
double mytime(){
  timerCounter++;
  ftime(&tp);
  return tp.time+((double)tp.millitm)/1000.0;
} 
#endif
#endif
void computeTimerOverhead(){
  const int n = 10000;
  minimaloverheadtime = 0.0;
  double sum = 0.0;
  for(int i=0;i<n;i++){
    double startwtime = mytime();
    double endwtime = mytime();
    double t = endwtime-startwtime;
    if(t > 0 &&(t < minimaloverheadtime || minimaloverheadtime == 0.0)){
      minimaloverheadtime=t;
    }
    sum += t;
  }
  averagedoverheadtime = sum/n;
  if(averagedoverheadtime < minimaloverheadtime)
    overheadtime = averagedoverheadtime;
  else
    overheadtime = (minimaloverheadtime+averagedoverheadtime)/2.0;
  timerCounter = 0;
}
long long timerCounter = 0;
double overheadtime = 0.0;
double averagedoverheadtime = 0.0;
double minimaloverheadtime = 0.0;

int  myId    = 0;
int  numProc = 1;
int  master  = 0;
bool periodic_boundary_x = false;
bool periodic_boundary_y = false;
bool periodic_boundary   = false;
ofstream debug;

#ifdef HAVE_MPI
MPI_Datatype TYPE_CELLINDEX;
MPI_Datatype TYPE_PARTICLE;
// One sided-communications, only for SGI
#if defined(_ABIN32) && defined(__sgi) && defined(HAVE_ONE)
int  MPI_Win_create(void *, MPI_Aint, int, MPI_Info, MPI_Comm, MPI_Win *){return 0;}
int PMPI_Win_create(void *, MPI_Aint, int, MPI_Info, MPI_Comm, MPI_Win *){return 0;}
int  MPI_Win_fence(int, MPI_Win){return 0;}
int PMPI_Win_fence(int, MPI_Win){return 0;}
int  MPI_Win_free(MPI_Win *){return 0;}
int PMPI_Win_free(MPI_Win *){return 0;}
int  MPI_Put(void *, int, MPI_Datatype, int, MPI_Aint, int, MPI_Datatype, MPI_Win){return 0;}
int PMPI_Put(void *, int, MPI_Datatype, int, MPI_Aint, int, MPI_Datatype, MPI_Win){return 0;}
int  MPI_Get(void *, int, MPI_Datatype, int, MPI_Aint, int, MPI_Datatype, MPI_Win){return 0;}
int PMPI_Get(void *, int, MPI_Datatype, int, MPI_Aint, int, MPI_Datatype, MPI_Win){return 0;}
#endif
#endif

#if !defined(HAVE_ONE)
extern void* myshmalloc(size_t size){
  return malloc(size);
}
extern void myshfree(void *ptr){
 if (ptr!=NULL) 
   free(ptr); 
}
#else
#if defined(__sgi) 
#include <mpp/shmem.h>
extern void* myshmalloc(size_t size){
  return shmalloc(size);
}
extern void myshfree(void *ptr){
  if (ptr!=NULL) 
   shfree(ptr); 
}
#else
extern void* myshmalloc(size_t size){
  void* ptr=NULL;
  MPI_Alloc_mem((MPI_Aint)(size),MPI_INFO_NULL,&ptr);
  return ptr;
}
extern void myshfree(void *ptr){
  if (ptr!=NULL) 
    MPI_Free_mem(ptr);
}
#endif
#endif
