/*
 * global.h
 *
 * Copyright (C) 1999-2000, Thierry Matthey <matthey@ii.uib.no>
 *
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 */

#ifndef _GLOBAL_H
#define _GLOBAL_H

#ifdef NO_ANSI_CPP
#include <fstream.h>
#else
#include <fstream>
using std::ofstream;
using std::ios;
#endif

#include <stdlib.h>
#include <values.h>
#ifdef HAVE_MPI
#include <mpi.h>
#endif
//_________________________________________________________________ global

// Real
typedef double Real;
#ifdef HAVE_MPI
#define TYPE_REAL MPI_DOUBLE
#endif

// Bool
#ifdef BOOL
typedef short bool;
#ifndef false
#define false (1==0)
#endif
#ifndef true
#define true (1==1)
#endif
#endif

// One sided-communications, only for SGI
#ifdef HAVE_MPI
#if defined(_ABIN32) && defined(__sgi) && defined(HAVE_ONE)

typedef unsigned int MPI_Win;
enum {
        MPI_WIN_NULL           = 0 
};
int  MPI_Win_create(void *, MPI_Aint, int, MPI_Info, MPI_Comm, MPI_Win *);
int PMPI_Win_create(void *, MPI_Aint, int, MPI_Info, MPI_Comm, MPI_Win *);
int  MPI_Win_fence(int, MPI_Win);
int PMPI_Win_fence(int, MPI_Win);
int  MPI_Win_free(MPI_Win *);
int PMPI_Win_free(MPI_Win *);
int  MPI_Put(void *, int, MPI_Datatype, int, MPI_Aint, int, MPI_Datatype, MPI_Win);
int PMPI_Put(void *, int, MPI_Datatype, int, MPI_Aint, int, MPI_Datatype, MPI_Win);
int  MPI_Get(void *, int, MPI_Datatype, int, MPI_Aint, int, MPI_Datatype, MPI_Win);
int PMPI_Get(void *, int, MPI_Datatype, int, MPI_Aint, int, MPI_Datatype, MPI_Win);
int  MPI_Accumulate(void *, int, MPI_Datatype, int, MPI_Aint, int, MPI_Datatype, MPI_Op, MPI_Win);
int PMPI_Accumulate(void *, int, MPI_Datatype, int, MPI_Aint, int, MPI_Datatype, MPI_Op, MPI_Win);
enum {
	MPI_MODE_NOCHECK	= 1,
	MPI_MODE_NOSTORE	= 2,
	MPI_MODE_NOPUT		= 4,
	MPI_MODE_NOPRECEDE	= 8,
	MPI_MODE_NOSUCCEED 	= 16
};
#endif
#endif


// Time
extern void computeTimerOverhead();
extern double averagedoverheadtime;
extern double overheadtime;
extern double minimaloverheadtime;
extern long long timerCounter;
#ifdef HAVE_MPI
//#define mytime() (MPI_Wtime())
extern double mytime();
#else
extern double mytime();
#endif

// Memory allocation/ freeing  
extern void* myshmalloc(size_t size);
extern void myshfree(void *ptr);

#define mymallocShm(type,nm)        myshmalloc(   nm*sizeof(type)+2*sizeof(long))
#define mymallocShmArray(type,nm,l) myshmalloc(l*(nm*sizeof(type)+2*sizeof(long)))
#define myfreeShm(ptr) {if (ptr!=NULL) myshfree(ptr); ptr = NULL;}

#define mymalloc(type,nm) malloc(nm*sizeof(type)+2*sizeof(long))
#define mymallocArray(type,nm,l) malloc(l*(nm*sizeof(type)+2*sizeof(long)))
#define myfree(ptr) {if (ptr!=NULL) free(ptr); ptr = NULL;}

#define mymallocsize(type,nm)        ((long)   (nm*sizeof(type)+2*sizeof(long)))
#define mymallocsizeArray(type,nm,l) ((long)(l*(nm*sizeof(type)+2*sizeof(long))))

//#define MAXDOUBLE 1.7976931348623157e+308
static const Real MAXReal =  MAXDOUBLE;
static const Real MINReal = -MAXDOUBLE;
static const Real EPSILON = 1e-30;
static const Real TWOTO31 = 0x7FFFFFF;


// Physical constants
static const Real FROM_ATOMIC_UNIT_TO_KG = 1.660565586E-27;
static const Real FROM_METER_TO_ANGSTROM = 1E10;
static const Real FROM_ANGSTROM_TO_METER = 1E-10;
static const Real K_BOLTZMANN            = 1.38066244E-23;
static const Real LENU                   = 1.0e-10;
static const Real FROM_EV_TO_J           = 1.602192e-19;
static const Real KBOLZ                  = 1.380662e-23/FROM_EV_TO_J;
// Max number of particles to display
static const long max_number_display = 50000;
// Max number of statvalues
static const long max_stat_values = 1000;
// Max size of filenames
static const long buffer_len = 200;
// Max number of diffrent atomtypes
static const long max_NTypes = 10;
// Max number of process
static const int max_number_proc = 128;
// Max number of neighbour proc's of a cell
static const long max_neighbours = 4;
// Boundary conditions
extern  bool periodic_boundary_x;
extern  bool periodic_boundary_y;
extern  bool periodic_boundary;
// Process id, number of process and master id
extern int myId;
extern int numProc; 
extern int master; 
// Global output files
extern ofstream debug;
extern ofstream output;

#ifdef HAVE_MPI
// MPI tags
static const int REQUEST_SRC_CELL    = 1;
static const int SEND_SRC_CELL       = 2;
static const int REQUEST_DEST_CELL   = 3;
static const int SEND_DEST_CELL      = 4;
static const int NOT_YET             = 5;
static const int REQUEST_BACK_CELL   = 6;
static const int SEND_BACK_CELL      = 7;
static const int NO_MORE_CELLS       = 8;
static const int SEND_BACK_ALL_CELLS = 9;
static const int SEND_INDEX          = 10;
static const int SEND_OTHER_CELL     = 11;
static const int REQUEST_PARTICLES   = 12;
static const int SEND_PARTICLES      = 13;
static const int NO_MORE_PARTICLES   = 14;
static const int UPDATE_CELLS        = 15;
static const int ALL_CELLS           = 16;
static const int LLA_CELLS           = 17;

// Derived types
extern MPI_Datatype TYPE_CELLINDEX;
extern MPI_Datatype TYPE_PARTICLE;
#endif

#endif
